//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"

#include "IniFiles.hpp"
#include "ChatContextData.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "IdCmdTCPServer"
#pragma link "IdCustomTCPServer"
#pragma link "IdScheduler"
#pragma link "IdSchedulerOfThread"
#pragma link "IdSchedulerOfThreadDefault"
#pragma resource "*.dfm"
TfrmMain *frmMain;
TIniFile *Ini;

//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnStartStopClick(TObject *Sender)
{
// This procedure should never change.
  if(ServerOnline)
  {
    StopServer();
  }
  else
  {
    StartServer();
  }
}
//---------------------------------------------------------------------------
bool __fastcall TfrmMain::CheckStartOK(void)
{
  int i = 0;
  for(int c=0; c<=lbIPs->Items->Count-1; c++)
  {
    if(lbIPs->Checked[c])
    {
      i++;
    }
  }
  bool bResult = i>0;
  if(!bResult)
  {
    Log("Can\'t start server until you select at least one IP to bind to.", clRed);
    MessageDlg("Can\'t start server until you select at least one IP to bind to.", mtError, TMsgDlgButtons() << mbOK, 0);
  }
// Add your code after this comment

  return bResult;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::PopulateIPAddresses(void)
{
// Again this section should not change
  lbIPs->Clear();
  lbIPs->Items = GStack->LocalAddresses;
  lbIPs->Items->Insert(0, "127.0.0.1");

  try
  {
    cbPorts->Items->Add(Iddsncoreresourcestrings_RSBindingAny);
    cbPorts->Items->BeginUpdate();
    TList *PortList = IdPorts();
    int i, port;
    for(i=0; i<PortList->Count;i++)
    {
      port = reinterpret_cast<int>(PortList->Items[i]);
      cbPorts->Items->Add(PortDescription(port));
    }
  }
  __finally
  {
    cbPorts->Items->EndUpdate();
  }
}
//---------------------------------------------------------------------------
AnsiString __fastcall TfrmMain::PortDescription(const int PortNumber)
{
// Guess what more code that shouldn't change
  AnsiString sResult = "";
  TStrings *tStr;

  tStr = ((TIdStackBSDBase*)GStack)->WSGetServByPort(PortNumber);
	if( tStr )
	{
		try
		{
			try
			{
				if( tStr->Count>0 )
				  sResult = AnsiString(PortNumber) + ": " + tStr->CommaText;
				else
				  sResult = PortNumber;
			}
			__finally
			{
				delete tStr;
			}
		}
		catch(const Exception &)
		{
		  sResult = PortNumber;
		}
	}
  return sResult;
}
//---------------------------------------------------------------------------
bool __fastcall TfrmMain::StartServer(void)
{
// This code starts the server up and posts back information about
// the server starting up.
// You should place your pre and post startup code in InternalServerBeforeStart
// and InternalServerAfterStart accordingly.
  TIdSocketHandle *Binding;
  bool bResult = false;
  if(!CheckStartOK())
  {
    return bResult;
  }

  TStringList *SL = new TStringList;
  if(!StopServer())
  {
    Log("Error stopping server", clRed);
    return false;
  }

  Server->Bindings->Clear();  // bindings cannot be cleared until TServer is inactive

  try
  {
    try
    {
      Server->DefaultPort = StrToInt(edPort->Text);
      for(int i=0; i!=lbIPs->Items->Count-1; i++)
      {
        if(lbIPs->Checked[i])
        {
          Binding = Server->Bindings->Add();
          Binding->IP = lbIPs->Items->Strings[i];
          Binding->Port = StrToInt(edPort->Text);
          Log( "Server bound to IP " + Binding->IP + " on port " + edPort->Text);
        }
      }

      if(InternalServerBeforeStart())
      {
        Server->Active = true;
        bResult = Server->Active;

        InternalServerAfterStart();
        if(ServerOnline)
        {
          Log("Server started", clGreen);
          btnStartStop->Caption = "Stop Server";
          SetControls();
        }
      }
    }
    catch(Exception &E)
    {
      Log("Server not started", clRed );
      Log( E.Message, clRed );
      bResult = false;
    }
  }
  __finally
  {
    delete SL;
  }
  return bResult;
}
//---------------------------------------------------------------------------
bool __fastcall TfrmMain::StopServer(void)
{
// This code stops the server and posts back information about
// the server shutting down.
// You should place your pre and post shutdown code in InternalServerBeforeStop
// and InternalServerAfterStop accordingly.
  bool bResult = false, b = Server->Active;

  if(InternalServerBeforeStop())
  {
    Server->Active = false;
    Server->Bindings->Clear();
    bResult = !Server->Active;

    if(bResult)
    {
      if(b)
      {
        Log("Server stopped", clGreen);
      }
    }
    else
    {
      Log( "Server not stopped", clRed );
    }
    InternalServerAfterStop();
    btnStartStop->Caption = "Start Server";
    SetControls();
  }
  else
  {
    Log( "Server not stopped", clRed );
  }
  return bResult;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
// Initialization routines.  You should find the appropriate procedure
// to initialize your stuff below.  The form create should hardly ever
// need to be changed.
  Ini = new TIniFile(ChangeFileExt(ParamStr(0), ".ini"));
  pcMain->ActivePageIndex = 0;
  pcMain->Align = alClient;
  PopulateIPAddresses();
  StopServer();
  LoadDefaultValues();
  CheckOptions();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormDestroy(TObject *Sender)
{
// If you created anything in form create or in one of the other
// initialization routines then get rid of it here.
  StopServer();
  SaveDefaultValues();
  delete Ini;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::LoadDefaultValues(void)
{
// This is were you get the chance to load values from the global
// Ini file.  The section for ports and IP's has been added in here
// for you by default.
  edPort->Text = Ini->ReadString("Settings", "Port", edPort->Text);
  int c = Ini->ReadInteger("Settings", "IPs", 0);
  for(int i=1; i<=c; i++)
  {
    AnsiString s = Ini->ReadString("Settings", "IP" + IntToStr(i), "");
    if(lbIPs->Items->IndexOf(s) > -1)
    {
      lbIPs->Checked[lbIPs->Items->IndexOf(s)] = true;
    }
  }
  c = Ini->ReadInteger("Greeting", "GreetingLines", 0);
  if(c > 0)
  {
    memGreeting->Lines->Clear();
    for(int i=0; i!=c-1; i++)
    {
      memGreeting->Lines->Add(Ini->ReadString("Greeting", "GreetingLine" + IntToStr(i), ""));
    }
    edUserPrompt->Text = Ini->ReadString("Greeting", "UsernamePrampt", "Username:");
  }
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::SaveDefaultValues(void)
{
// This is were you get the chance to save values to the global
// Ini file.  The section for ports and IP's has been added in here
// for you by default.
  int i, c=0;
  Ini->WriteString("Settings", "Port", edPort->Text);
  for(i=0; i<lbIPs->Items->Count; i++)
  {
    if(lbIPs->Checked[i])
    {
      c++;
      Ini->WriteString("Settings", "IP" + IntToStr(c), lbIPs->Items->Strings[i]);
    }
  }
  Ini->WriteInteger("Settings", "IPs", c);
  Ini->WriteInteger("Placement", "Top", Top);
  Ini->WriteInteger("Placement", "Left", Left);

  Ini->WriteInteger("Greeting", "GreetingLines", memGreeting->Lines->Count);
  for(i=0; i<=memGreeting->Lines->Count-1; i++)
  {
    Ini->WriteString("Greeting", "GreetingLine" + IntToStr(i), memGreeting->Lines->Strings[i]);
  }
  Ini->WriteString("Greeting", "UsernamePrompt", edUserPrompt->Text);
}
//---------------------------------------------------------------------------
AnsiString OptName(AnsiString opt)
{
  AnsiString sResult;
  if(opt.Pos("=")>0)
  {
    sResult = opt.SubString(1, opt.Pos("=" - 1));
    if(sResult[1] == '-' || sResult[1] == '/'|| sResult[1] == '\\')
    {
      sResult = sResult.SubString(2,sResult.Length());
    }
  }
  else
  {
    sResult = opt;
  }
  return sResult;
};

AnsiString OptValue(AnsiString opt)
{
  AnsiString sResult;
  if(opt.Pos("=") > 0)
  {
    sResult = opt.SubString(opt.Pos("=" + 1), opt.Length());
  }
  else
  {
    sResult = opt;
  }
  return sResult;
};

void __fastcall TfrmMain::CheckOptions(void)
{
// The check options procedure should be used to check commandline options
// if you wish to support command line options then please add it here.
// By default port and autostart are supported.
  int i;
  AnsiString opt;
  bool bDoAutoStart = false;

  for(i=1; i <= ParamCount(); i++)
  {
    opt = LowerCase(ParamStr(i));
    if(OptName(opt) == "port")
    {
      edPort->Text = OptValue(opt);
    }

    if(OptName(opt) == "autostart")
    {
      bDoAutoStart = true;
    }
  }

  if(bDoAutoStart)
  {
    StartServer();
  }
}
//---------------------------------------------------------------------------
bool __fastcall TfrmMain::GetServerOnline(void)
{
// Just a faster way then checking server.active for some
  return Server->Active;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::lbProcessesDrawItem(TWinControl *Control,
      int Index, TRect &Rect, TOwnerDrawState State)
{
// This draws the items in the Process Log in colors to allow quick
// visual inspection
  TListBox *lb = (TListBox*)Control;
  lb->Canvas->Brush->Color = lb->Color;
  lb->Canvas->FillRect(Rect);
  lb->Canvas->Font->Color = (TColor)lb->Items->Objects[Index];
  lb->Canvas->TextOut(Rect.left + 2, Rect.top, lb->Items->Strings[Index]);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::Log(AnsiString Msg, TColor Color)
{
// Simply adds a new item to the process log and then makes it the
// currently selected item.
  lbProcesses->Items->AddObject(Msg, (TObject*)Color);
  lbProcesses->ItemIndex = lbProcesses->Items->Count - 1;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ServerStatus(TObject *ASender,
      const TIdStatus AStatus, const AnsiString AStatusText)
{
// Logs any ServerStatus messages to the Process Log
  Log(AStatusText);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ServerException(TIdContext *AContext,
      Exception *AException)
{
// Logs any server exceptions to the Process Log
  Log(AException->Message, clRed);
}
//---------------------------------------------------------------------------
bool __fastcall TfrmMain::InternalServerBeforeStart(void)
{
  // Preform your startup code here.  If you do not wish the server to start
  // then simply return false from this function and report back the proper
  // error by calling Log(YourMessage, clRed);
  try
  {
    Server->Greeting->Text->Assign(memGreeting->Lines);
    return true;
  }
  catch(Exception &E)
  {
    Log(E.Message, clRed);
    return false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::InternalServerAfterStart(void)
{
// Your code should go here.  At this point the server is active.
// So if you need to stop it then you should call StopServer
// or for a hard halt call Server.Active := false;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::InternalServerAfterStop(void)
{
// Your code should go here.  At this point the server has been stoped.
// So if you need to start it then you should call StartServer
// or for a force start call Server.Active := true;
}
//---------------------------------------------------------------------------
bool __fastcall TfrmMain::InternalServerBeforeStop(void)
{
  // Preform your shutdown code here.  If you do not wish the server to stop
  // then simply return false from this function and report back the proper
  // error by calling Log(YourMessage, clRed);
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::SetControls(void)
{
// Sets up the UI controls to either be enabled or disabled based upon
// the current server state.  See below for examples.
  lbIPs->Enabled         = !ServerOnline;
  edPort->Enabled        = !ServerOnline;
  cbPorts->Enabled       = !ServerOnline;
  memGreeting->Enabled   = !ServerOnline;
  edUserPrompt->Enabled  = !ServerOnline;
  btnTestClient->Enabled = ServerOnline;

}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormActivate(TObject *Sender)
{
  Top = Ini->ReadInteger("Placement", "Top", Top);
  Left= Ini->ReadInteger("Placement", "Left", Left);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ServerExecute(TIdContext *AContext)
{
// Your stuff for OnExecute goes here.
  ((TChatContextData*)(AContext->Data))->CheckMsg(AContext);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ServerConnect(TIdContext *AContext)
{
  AnsiString s;
  AContext->Data = new TChatContextData;
  ((TChatContextData*)AContext->Data)->OnMsgAvail = &MsgAvail;
  s = edUserPrompt->Text + char(13) + char(10);
  TIdIOHandler* IOH = AContext->Connection->IOHandler;
  IOH->WriteLn(s);

  TIdSocketHandle *ISH = AContext->Binding();
  Log("Client connection established from ip: " + ISH->PeerIP, clBlue);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ServerDisconnect(TIdContext *AContext)
{
  Log("Client connection removed from ip: " + AContext->Binding()->PeerIP, clBlue);

  AContext->Data->Free();
  AContext->Data = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::edPortKeyPress(TObject *Sender, char &Key)
{
  bool bResult;
  switch(Key)
  {
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
    case 0x08:
      bResult = true;
    default:
      bResult = false;
  }

  if(!bResult)
    Key = 0x00;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnTestClientClick(TObject *Sender)
{
  ShellExecute(Handle, "OPEN", "Telnet",
               PChar(("localhost "+IntToStr(Server->DefaultPort)).c_str()),
               NULL, SW_SHOWNORMAL);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::WriteMessage(AnsiString Msg)
{
  TList *cList;
  int Count;
  TIdContext *LContext;

  cList = Server->Contexts->LockList();
  try
  {
    for(Count=0; Count!=cList->Count; Count++)
    {
      LContext = (TIdContext* )(cList->Items[Count]);
      LContext->Connection->IOHandler->WriteLn(Msg);
    }
  }
  __finally
  {
    Server->Contexts->UnlockList();
  }
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::MsgAvail(TChatContextData *Sender)
{
  AnsiString s;
  s = Sender->Pop();

  s = s+char(10)+char(13);
  WriteMessage(s);
}
//---------------------------------------------------------------------------

